function result = createGraph(modeltype, distP1, distP2, sizeDist, sizeRange,...
    avgDensityInside, avgDensityOutside, targetNumCommunities, assembleGraph)
%% CREATEGRAPH creates a random graph with hyperbolic community structure
%  Takes the distribution functions (i.e. Matlab distribution objects, as 
%  obtained from fitdist() or makedist()) for the model parameters of the 
%  specified hyperbolic model ('H,gamma', 'p,Theta', or 'x,Sigma') as well 
%  as the distribution of community sizes, and the observed densities. 
%  The output is a new graph on the basis of these parameters. 
%  The number of communities in the new graph is targetNumCommunities, and
%  the sizes of the communities range within the specified sizeRange. 
%  The result is the adjacency matrix of the newly generated graph if 
%  assembleGraph=true. Otherwise, the collection of communities will be 
%  returned, each with their hyperbolic model (parameterized by 'H,gamma')
%  and a list of global node indices.
%
%  Distribution functions for gamma need to range on gamma, relative to
%  community size, i.e. between 0 and 1, scaled by factor 1000. Hence,
%  values between 0 and 1000 are expected.
%  Distribution functions for H need to range between 1 and 1001, and will
%  be interpreted as fractions of the core size after dividing by 1000.
%
    l = targetNumCommunities;
    lengths = round(random(truncate(sizeDist,sizeRange(1),sizeRange(2)),l,1));
    [Hs, gammas] = getHGammaFromDist(modeltype, distP1, distP2, lengths, l);

    isValid = arrayfun(@isValidParameterConfiguration,lengths,Hs,gammas);
    while ~all(isValid)
        toRedraw = find(~isValid);
        r = length(toRedraw);
        %fprintf('\t we need to re-draw %i samples\n', r);
        lengths(toRedraw) = round(random(truncate(sizeDist,100,1000), r,1));        
        [Hs(toRedraw), gammas(toRedraw)] = getHGammaFromDist(modeltype, ...
            distP1, distP2, lengths(toRedraw), r);

        isValid(toRedraw) = arrayfun(@isValidParameterConfiguration, ...
            lengths(toRedraw), Hs(toRedraw), gammas(toRedraw));
    end
            
    offsets = [1; cumsum(lengths(1:end-1))+1];
    comms = [];
    for i=1:l
        %fprintf('.');
        comms(i).members=offsets(i):offsets(i)+lengths(i)-1;
        comms(i).paramType='H,gamma';
        c = makeCommunity(lengths(i), Hs(i), gammas(i), getCondition(comms(i).paramType));
        
        % apply sparsity/noise
        posOfOnes = find(triu(c));
        posOfZeros = find(triu(~c));
        stayOne = binornd(1,avgDensityInside,length(posOfOnes),1);
        noisePos = binornd(1,avgDensityOutside,length(posOfZeros),1);
        c(posOfOnes(~stayOne)) = 0;
        c(posOfZeros(noisePos==1)) = 1;
        c = triu(c) | transpose(triu(c));
        
        comms(i).comm = c;
        comms(i).p1 = Hs(i);
        comms(i).p2 = gammas(i);
        % for compatibility with the output of HyBoBo code
        comms(i).LL = -Inf;  
        comms(i).model = [];
    end
    if assembleGraph
        generatedGraph = sparse(sum(lengths), sum(lengths));
        % noise outside the communities
        posOfZeros = find(triu(~generatedGraph));
        noisePos = binornd(1,avgDensityOutside,length(posOfZeros),1);
        generatedGraph(posOfZeros(noisePos==1)) = 1;
        generatedGraph = triu(generatedGraph) | transpose(triu(generatedGraph));
        for i=1:l
            generatedGraph(comms(i).members,comms(i).members) = comms(i).comm; % non-overlapping communities
        end
        result = generatedGraph;
    else
        result = comms;
    end
end

function [Hs, gammas] = getHGammaFromDist(modeltype, distP1, distP2, lengths, numSamples)
    switch modeltype
        case 'H,gamma'
            gammas = round(random(distP2, numSamples,1) ./ 1000 .* lengths);
            Hs = round((random(truncate(distP1, 1,1001), numSamples,1) -1)./ 1000 .* (gammas)); %go back to also include H=0
        case 'p,Theta'
            ps = random(distP1, numSamples,1);
            Thetas = random(distP2, numSamples,1);
            Hs = arrayfun(@getH, Thetas,ps,lengths);
            gammas = arrayfun(@getGamma, ps,Hs,lengths);
        case 'x,Sigma'
            xs = random(truncate(distP1, 0,1), numSamples,1);
            Sigmas = random(distP2, numSamples,1);
            ps = arrayfun(@getPFromX, xs);
            Thetas = arrayfun(@getThetaFromP, ps, Sigmas);
            Hs = arrayfun(@getH, Thetas,ps,lengths);
            gammas = arrayfun(@getGamma, ps,Hs,lengths);
        otherwise
            error('The modeltype is not valid.');
    end
    % we admit only integer values for H and gamma in the computation
    Hs = round(Hs);
    gammas = round(gammas);
end