% generates random graphs with hyperbolic community structure and evaluates
% how close they match the original observations on the provided datasets
dataName={'youtube', 'amazon','dblp',  'friendster'};
distFilesPath = 'Data/paramDist_gammaH_';
storeFolder = 'Results/';
perfectModel = 0;
skipExisting = 1;
numSamples = 100;
sizeVariation = 1; % don't vary for the conditional entropy test

for dataNameIndex=1:length(dataName) 
    d = char(dataName(dataNameIndex));
    disp(['--------' d '-------']);
    
    disp('read the fitted models');
    f = [distFilesPath d '.csv']; 
    dataArray = textscan(fopen(f,'r'), '%f%f%f%f%f%f%f%[^\n\r]', 'Delimiter', ',',  'ReturnOnError', false);
    coreSize = dataArray{:,2};
    relCoreSize = dataArray{:,2}./dataArray{:,3};
    relCoreSizeAsInteger = round(relCoreSize * 1000);
    relTailSize = (dataArray{:,1})./(dataArray{:,2}); 
    relTailSizeAsInteger = round(relTailSize * 1000) + 1; %let's move this to strictly positive as the dist doesn't like 0s
    averageInsideDensity = mean(dataArray{:,4}./dataArray{:,6}); 
    averageOutsideDensity = mean(dataArray{:,5}./dataArray{:,7});
    l = length(coreSize);
    
    disp('fit a distribution for gamma and H');
    gammaDist = fitdist(relCoreSizeAsInteger(relCoreSizeAsInteger>0),'Normal');
    HDist = fitdist(relTailSizeAsInteger(relTailSizeAsInteger>0),'Exponential');
    sizeDist = fitdist(dataArray{:,3},'GeneralizedExtremeValue');
    
    for sample = 1:numSamples
        if skipExisting && exist([storeFolder d '_' num2str(sample) '.mat'], 'file') == 2
            continue;
        end
        disp('make graph with these distributions of parameters');
        if sizeVariation
            lengths = round(random(truncate(sizeDist,100,1000), l,1));
        else
            lengths = dataArray{:,3};% - take away this uncertainty for computing the conditional entropy 
        end
        gammas = round(random(gammaDist, l,1) ./ 1000 .* lengths);
        Hs = round((random(truncate(HDist, 1,1001), l,1) -1)./ 1000 .* (gammas)); %go back to also include H=0
        
        isValid = arrayfun(@isValidParameterConfiguration,lengths,Hs,gammas);
        while ~all(isValid)
            toRedraw = find(~isValid);
            r = length(toRedraw);
            fprintf('\t we need to re-draw %i samples\n', r);
            if sizeVariation
                lengths(toRedraw) = round(random(truncate(sizeDist,100,1000), r,1));
            end
            gammas(toRedraw) = round(random(gammaDist, r,1) ./ 1000 .* lengths(toRedraw));
            Hs(toRedraw) = round((random(truncate(HDist, 1,1001), r,1) -1)./ 1000 .* (gammas(toRedraw)));

            isValid(toRedraw) = arrayfun(@isValidParameterConfiguration, lengths(toRedraw), Hs(toRedraw), gammas(toRedraw));
        end
            
        offsets = [1; cumsum(lengths(1:end-1))+1];
        comms = [];
        for i=1:l
            fprintf('.');
            comms(i).members=offsets(i):offsets(i)+lengths(i)-1;
            comms(i).paramType='H,gamma';
            c = makeCommunity(lengths(i), Hs(i), gammas(i), getCondition(comms(i).paramType));
            if ~perfectModel % apply sparsity/noise
                posOfOnes = find(triu(c));
                posOfZeros = find(triu(~c));
                stayOne = binornd(1,averageInsideDensity,length(posOfOnes),1);
                noisePos = binornd(1,averageOutsideDensity,length(posOfZeros),1);
                c(posOfOnes(~stayOne)) = 0;
                c(posOfZeros(noisePos==1)) = 1;
                c = triu(c) | transpose(triu(c));
            end
            comms(i).comm = c;
            comms(i).p1 = Hs(i);
            comms(i).p2 = gammas(i);
            comms(i).LL = -Inf;
            comms(i).model = [];
            
        end
        fprintf('\n');
        disp( 'and fit the hyperbolic model on the generated graph');
        parfor i = 1:l
            fprintf('.');
            [ comms(i).p1, comms(i).p2, comms(i).LL ] = recoverCommunity(comms(i).comm, comms(i).paramType);
            comms(i).model = makeCommunity(lengths(i), comms(i).p1, comms(i).p2, getCondition(comms(i).paramType));
        end
        fprintf('\n');

        disp('store result');
        save([storeFolder d '_' num2str(sample)], 'comms', 'gammaDist', 'HDist', 'sizeDist');
    end
end